
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <time.h>
#include "input_source.h"


static unsigned Prompt ( const char * prompt , bool yes_no , unsigned max_num )
{
    char        buf [ 64 ] ;

    while ( 1 )
    {
        fputs ( prompt , stdout ) ;
        if ( yes_no )
        {
            fputs ( " ('yes' or 'no'): " , stdout ) ;
        } else {
            printf ( " (0 to %u): " , max_num ) ;
        }
        fflush ( stdout ) ;
        fgets ( buf , 60 , stdin ) ;
        buf [ 59 ] = '\0' ;
        
        if ( yes_no )
        {
            buf [ 0 ] = toupper ( buf [ 0 ] ) ;
            if ( buf [ 0 ] == 'Y' )
            {
                return 1 ;
            } else if ( buf [ 0 ] == 'N' )
            {
                return 0 ;
            }
        } else {
            char *      endptr ;
            long int    v = strtol ( buf , & endptr , 10 ) ;

            if (( v >= 0 )
            && ( v <= (long int) max_num )
            && ( endptr != buf ))
            {
                return (unsigned) v ;
            }
        }
    }
}

void Main ( void )
{
    Input_Source    input_source ;
    Mixer_Info *    mix ;
    unsigned        csrc , i , input_lock ;

    printf ( 
"Sound Troll version 1.0\n\n"
"This program stops other programs changing your sound input settings.\n" ) ;

    if ( input_source . mixer . size () > 1 )
    {
        printf ( "\nWhich sound card is your microphone attached to?\n" ) ;
        for ( i = 0 ; i < input_source . mixer . size () ; i ++ )
        {
            mix = input_source . mixer [ i ] ;
            printf ( "Card %u: %s\n" , i , mix -> name . c_str () ) ;
        }
        i = Prompt ( "Enter the card number: " , false , 
                    (unsigned) ( input_source . mixer . size () - 1 )) ;

        if ( i >= input_source . mixer . size () ) return ;
    
        mix = input_source . mixer [ i ] ;
    } else if ( input_source . mixer . size () == 1 )
    {
        mix = input_source . mixer [ 0 ] ;
    } else {
        printf ( "No mixers detected. Sorry.\n" ) ;
        return ;
    }

	bool check = true ;
    while ( check )
    {
        input_lock = mix -> Get_Current_Source () ;
        if ( input_lock >= mix -> source_name . size () )
        {
            printf ( "\nMixer \"%s\" is not set to any input.\n" ,
                    mix -> name . c_str () ) ;
        } else {
            printf ( "\nMixer \"%s\" is currently set to input %u: \"%s\"\n" ,
                    mix -> name . c_str () ,
                    input_lock , mix -> source_name [ input_lock ] . c_str () ) ;
            if ( Prompt ( "Is this the setting you want to keep?" , true , 0 ) )
            {
                check = false ;
            }
			printf ( "\n" ) ;
        }
        if ( check )
        {
            printf ( "Please use the Windows volume control to set the mixer "
                    "to the input you want.\n" ) ;
            if ( ! Prompt ( "Read the settings again?" , true , 0 ) )
            {
                return ;
            }
        }
    }


	printf ( "Sound Troll will keep mixer \"%s\"\n"
			"locked to input %u: \"%s\"\n" ,
            mix -> name . c_str () , input_lock ,
			mix -> source_name [ input_lock ] . c_str () ) ;
    printf ( "Close the window or press Control-C to exit.\n\n" ) ;
    srand ( (unsigned) time ( NULL ) ) ;

    while ( 1 )
    {
        csrc = mix -> Get_Current_Source () ;
        if ( csrc != input_lock )
        {
            if ( csrc < mix -> source_name . size () )
            {
                printf ( "Input was changed to %u: \"%s\". Changing it back.\n" ,
                    csrc , mix -> source_name [ csrc ] . c_str () ) ;
            }
            mix -> Set_Current_Source ( input_lock ) ;
            Sleep ( 1000 ) ;
        } else {
            /* Sleep for 1..6 seconds before testing it again */
            Sleep ( 1000 + (unsigned) (( rand () >> 8 ) % 5000 )) ;
        }
    }
}


