
#include <stdio.h>
#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif


typedef struct px_mixer_lite {
    void *info;
    } px_mixer ;

typedef struct PxCtrl
{
   char  *name;
   DWORD lineID;
   DWORD controlID;
} PxCtrl;

typedef struct PxInfo
{
   HMIXEROBJ   hInputMixer;
   char        *inName;
   int         numInputs;
   PxCtrl      *src;

   DWORD       muxID;
} PxInfo;

static int cleanup(px_mixer *Px) ;

static UINT get_ctrls(HMIXEROBJ mixer, DWORD lineID, PxCtrl **pctrls)
{
   MMRESULT res;
   MIXERLINE line;
   MIXERLINECONTROLS controls;
   MIXERCONTROL control;
   PxCtrl *ctrls = NULL;
   UINT num;
   UINT s;

   do {
      line.cbStruct = sizeof(MIXERLINE);
      line.dwComponentType = lineID;

      res = mixerGetLineInfo(mixer,
                             &line,
                             MIXER_GETLINEINFOF_COMPONENTTYPE);

      if (res != MMSYSERR_NOERROR) {
         break;
      }

      num = (UINT) line.cConnections;

      ctrls = calloc(num, sizeof(PxInfo));
      if (ctrls == NULL) {
         break;
      }

      for (s = 0; s < num; s++)
      {
         line.dwSource      = s;

         res = mixerGetLineInfo(mixer,
                                &line,
                                MIXER_GETLINEINFOF_SOURCE);

         if (res != MMSYSERR_NOERROR) {
            break;
         }

         ctrls[s].lineID = line.dwLineID;
         ctrls[s].name = strdup(line.szName);
         if (ctrls[s].name == NULL) {
            break;
         }

         controls.cbStruct = sizeof(MIXERLINECONTROLS);
         controls.dwLineID = line.dwLineID;
         controls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
         controls.cbmxctrl = sizeof(MIXERCONTROL);
         controls.pamxctrl = &control;

         control.cbStruct = sizeof(MIXERCONTROL);

         res = mixerGetLineControls(mixer,
                                    &controls,
                                    MIXER_GETLINECONTROLSF_ONEBYTYPE);

         if (res == MMSYSERR_NOERROR)
            ctrls[s].controlID = control.dwControlID;
         else
            ctrls[s].controlID = 0;
      }

      if (s != num) {
         break;
      }

      *pctrls = ctrls;

      return num;

   } while (FALSE);

   if (ctrls) {
      for (s = 0; s < num; s++) {
         if (ctrls[s].name) {
            free(ctrls[s].name);
         }
      }

      free(ctrls);
   }

   return 0;
}

static DWORD find_ctrl(HMIXEROBJ mixer, DWORD lineID, DWORD ctrlID)
{
   MMRESULT res;
   MIXERLINE line;
   MIXERLINECONTROLS controls;
   MIXERCONTROL control;

   line.cbStruct = sizeof(MIXERLINE);
   line.dwComponentType = lineID;

   res = mixerGetLineInfo(mixer,
                          &line,
                          MIXER_GETLINEINFOF_COMPONENTTYPE);

   if (res != MMSYSERR_NOERROR) {
      return -1;
   }

   controls.cbStruct = sizeof(MIXERLINECONTROLS);
   controls.dwLineID = line.dwLineID;
   controls.dwControlType = ctrlID;
   controls.cbmxctrl = sizeof(MIXERCONTROL);
   controls.pamxctrl = &control;

   control.cbStruct = sizeof(MIXERCONTROL);

   res = mixerGetLineControls(mixer,
                              &controls,
                              MIXER_GETLINECONTROLSF_ONEBYTYPE);

   if (res != MMSYSERR_NOERROR) {
      return -1;
   }

   return control.dwControlID;
}

static int open_mixers(px_mixer *Px, UINT deviceIn)
{
   PxInfo*info;
   MMRESULT res;

   if (deviceIn == UINT_MAX) {
      return FALSE;
   }

   Px->info = calloc(1, sizeof(PxInfo));
   if (Px->info == NULL) {
      return FALSE;
   }

   info = (PxInfo *) Px->info;
   info->hInputMixer = NULL;
   info->numInputs = 0;
   info->muxID = 0;

   if (deviceIn != UINT_MAX) {
      res = mixerOpen((LPHMIXER) &info->hInputMixer,
                      deviceIn,
                      0,
                      0,
                      MIXER_OBJECTF_MIXER);
      if (res != MMSYSERR_NOERROR) {
         return cleanup(Px);
      }

      info->muxID = find_ctrl(info->hInputMixer,
                              MIXERLINE_COMPONENTTYPE_DST_WAVEIN,
                              MIXERCONTROL_CONTROLTYPE_MUX);
      if (info->muxID == -1) {
         info->muxID = find_ctrl(info->hInputMixer,
                                 MIXERLINE_COMPONENTTYPE_DST_WAVEIN,
                                 MIXERCONTROL_CONTROLTYPE_MIXER);
      }

      info->numInputs = get_ctrls(info->hInputMixer,
                                  MIXERLINE_COMPONENTTYPE_DST_WAVEIN,
                                  &info->src);

      if (info->numInputs == 0) {
         return cleanup(Px);
      }
   }
   return TRUE;
}

static int cleanup(px_mixer *Px)
{
   PxInfo *info = (PxInfo *)Px->info;
   int i;

   if (info) {
      if (info->hInputMixer) {
         mixerClose((HMIXER) info->hInputMixer);
      }
   
      if (info->inName) {
         free(info->inName);
      }

      if (info->src) {
         for (i = 0; i < info->numInputs; i++) {
            if (info->src[i].name) {
               free(info->src[i].name);
            }
         }

         free(info->src);
      }

      free(info);
      Px->info = NULL;
   }

   return FALSE;
}

static void close_mixer(px_mixer *Px)
{
   PxInfo *info = (PxInfo *)Px->info;

   cleanup(Px);
}
/*
|| Input source
*/

static int get_num_input_sources(px_mixer *Px)
{
   PxInfo *info = (PxInfo *)Px->info;

   return info->numInputs;
}

static const char *get_input_source_name(px_mixer *Px, int i)
{
   PxInfo *info = (PxInfo *)Px->info;

   if (i < info->numInputs) {
      return info->src[i].name;
   }

   return NULL;
}

static int get_current_input_source(px_mixer *Px)
{
   PxInfo *info = (PxInfo *)Px->info;
   MIXERCONTROLDETAILS details;
   MIXERCONTROLDETAILS_LISTTEXT *list;
   MIXERCONTROLDETAILS_BOOLEAN *flags;
   MMRESULT res;
   int i = -1;
   int j;

   if (!info->hInputMixer) {
      return -1;
   }

   do {
      list = calloc(info->numInputs, sizeof(MIXERCONTROLDETAILS_LISTTEXT));
      flags = calloc(info->numInputs, sizeof(MIXERCONTROLDETAILS_BOOLEAN));
      if (list == NULL || flags == NULL) {
         break;
      }

      details.cbStruct = sizeof(MIXERCONTROLDETAILS);
      details.dwControlID = info->muxID;
      details.cMultipleItems = info->numInputs;
      details.cChannels = 1;   
      details.cbDetails = sizeof(MIXERCONTROLDETAILS_LISTTEXT);
      details.paDetails = &list[0];

      res = mixerGetControlDetails(info->hInputMixer,
                                   &details,
                                   MIXER_GETCONTROLDETAILSF_LISTTEXT);

      if (res != MMSYSERR_NOERROR) {
         break;
      }

      details.cbStruct = sizeof(MIXERCONTROLDETAILS);
      details.dwControlID = info->muxID;
      details.cMultipleItems = info->numInputs;
      details.cChannels = 1;   
      details.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
      details.paDetails = &flags[0];

      res = mixerGetControlDetails(info->hInputMixer,
                                   &details,
                                   MIXER_SETCONTROLDETAILSF_VALUE);

      if (res != MMSYSERR_NOERROR) {
         break;
      }

      for (j = 0; j < info->numInputs; j++) {
         if (flags[j].fValue) {
            i = j;
            break;
         }
      }

      for (j = 0; j < info->numInputs; j++) {
         if (info->src[j].lineID == list[i].dwParam1 ) {
            i = j;
            break;
         }
      }

   } while(FALSE);

   if (list) {
      free(list);
   }

   if (flags) {
      free(flags);
   }

   return i;
}

static void set_current_input_source(px_mixer *Px, int i)
{
   PxInfo *info = (PxInfo *)Px->info;
   MIXERCONTROLDETAILS details;
   MIXERCONTROLDETAILS_LISTTEXT *list;
   MIXERCONTROLDETAILS_BOOLEAN *flags;
   MMRESULT res;
   int j;

   if (!info->hInputMixer) {
      return;
   }

   do {
      list = calloc(info->numInputs, sizeof(MIXERCONTROLDETAILS_LISTTEXT));
      flags = calloc(info->numInputs, sizeof(MIXERCONTROLDETAILS_BOOLEAN));
      if (list == NULL || flags == NULL) {
         break;
      }

      details.cbStruct = sizeof(MIXERCONTROLDETAILS);
      details.dwControlID = info->muxID;
      details.cMultipleItems = info->numInputs;
      details.cChannels = 1;   
      details.cbDetails = sizeof(MIXERCONTROLDETAILS_LISTTEXT);
      details.paDetails = &list[0];

      res = mixerGetControlDetails(info->hInputMixer,
                                   &details,
                                   MIXER_GETCONTROLDETAILSF_LISTTEXT);

      if (res != MMSYSERR_NOERROR) {
         break;
      }

      for (j = 0; j < info->numInputs; j++) {
         if (list[j].dwParam1 == info->src[i].lineID) {
            flags[j].fValue = TRUE;
            break;
         }
      }

      details.cbStruct = sizeof(MIXERCONTROLDETAILS);
      details.dwControlID = info->muxID;
      details.cMultipleItems = info->numInputs;
      details.cChannels = 1;   
      details.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
      details.paDetails = &flags[0];

      res = mixerSetControlDetails(info->hInputMixer,
                                   &details,
                                   MIXER_SETCONTROLDETAILSF_VALUE);

      if (res != MMSYSERR_NOERROR) {
         break;
      }
   } while(FALSE);

   if (list) {
      free(list);
   }

   if (flags) {
      free(flags);
   }

   return;
}



void Test ( void )
{
    px_mixer    Px [ 1 ] ;
    UINT        uMxId , num_mixers , rc , src , deviceIn ;
    MIXERCAPS   mxcaps ;

    num_mixers = mixerGetNumDevs () ;
    printf ( "mixerGetNumDevs () == %u\n" , num_mixers ) ;

    for ( uMxId = 0 ; uMxId < num_mixers ; uMxId ++ )
    {
        rc = mixerGetDevCaps ( uMxId , & mxcaps , sizeof ( MIXERCAPS ) ) ;
        printf ( "mixerGetDevCaps ( %u ) == %u" , uMxId , rc ) ;
        if ( rc != MMSYSERR_NOERROR ) continue ;

        printf ( " name \"%s\"\n" , mxcaps . szPname ) ;

		deviceIn = 0 ;
		rc = mixerGetID ( (HMIXEROBJ) uMxId , 
                    & deviceIn , MIXER_OBJECTF_MIXER ) ;
        printf ( " mixerGetId ( %u ) == %u devicein = %u\n" , 
                    uMxId , rc , deviceIn ) ;
		if ( rc != MMSYSERR_NOERROR ) continue ;

        rc = open_mixers ( Px , deviceIn ) ;
        printf ( " open_mixers ( %u ) == %u\n" , uMxId , rc ) ;

        if ( ! rc ) continue ;
            
        rc = get_num_input_sources ( Px ) ;
        printf ( "  get_num_input_sources () == %u\n" , rc ) ;
        for ( src = 0 ; src < rc ; src ++ )
        {
            printf ( "   name [ %u ] = %s\n" , src ,
                    get_input_source_name ( Px , src ) ) ;
        }
        printf ( "   current input = %u\n" , 
                    get_current_input_source ( Px ) ) ;
        close_mixer ( Px ) ;
    }
}


#ifdef __cplusplus
}
#endif

